
//
// MoonApp.java -- display the moon in its current phase.
//	Copyright 1996 by Akkana Peck.
// You may use or distribute this code under the terms of the GPL.
//

package Akk.Astro;

import java.awt.*;
import java.applet.*;
import java.util.Date;

public class MoonApp extends Applet
{
    MoonPhase moon;

    boolean showEarthshine;

    // Colors:
    Color moonColor;
    Color earthshineColor;

     // Copies of the UI components we maintain:
    Canvas drawingArea;
    TextField dateField;
    Choice invertMenu;
    Panel controlPanel;
    int borderWidth;

    public String getAppletInfo() {
	return "Phase of the Moon.\nCopyright 1996 by Akkana Peck";
    }

    public String[][] getParameterInfo() {
	String[][] info = {
	    { "time", "double",
	      "Time of chart (optional, defaults to current)" },
	    { "showEarthshine", "boolean",
	      "Show the dark portion of the moon in grey" },
	    { "showDate", "Boolean", "Display date, or not" },
	    { "borderWidth", "Integer", "width of the white border (default 0)" },
   	};
	return info;
    }

    public void init()
    {
	//
	// Get the parameters:
	//
    	String time = getParameter("time");

    	String s = getParameter("showEarthshine");
	if (s != null)
	    showEarthshine = Boolean.valueOf(s).booleanValue();
	else
	    showEarthshine = true;

    	s = getParameter("showDate");
	boolean showDate;
	if (s != null)
	    showDate = Boolean.valueOf(s).booleanValue();
	else
	    showDate = true;

	s = getParameter("borderWidth");
	if (s != null)
	    borderWidth = Integer.valueOf(s).intValue();
	else
	    borderWidth = 0;

	//
	// Set some layout info:
	//
	setLayout(new BorderLayout(0, 1));

	//
	// Create the star chart object
	//
	drawingArea = new Canvas();
	add("North", drawingArea);

	StarDate now = null;
        try {
            if (time == null)
                now = new StarDate();
            else
                now = new StarDate(now);

            dateField = new TextField(now.toString(), 12);
            if (showDate)
                add("South", dateField);
            // else maintain the component, just don't show it.
        }
        catch (IllegalArgumentException e) {
            dateField = new TextField(now.toString(), 12);
        }
    }

    public boolean handleEvent(Event event)
    {
	if (event.id == Event.ACTION_EVENT && event.target == dateField)
	{
            repaint();
	}
	return super.handleEvent(event);
    }

    public void paint(Graphics g)
    {
    	int width = size().width;
        int height = size().height - dateField.size().height;

        StarDate date;
        try {
            date = new StarDate(dateField.getText());
        }
        catch (IllegalArgumentException e) {
            g.setColor(Color.black);
            g.fillRect(0, 0, width, height);
            g.setColor(Color.yellow);
            g.drawString("Error: can't parse date " + dateField.getText(),
                         5, height/2);
            return;
        }

    	if (moon == null)
    	    moon = new MoonPhase();
	double phaseAngle = moon.getPhaseAngle(date);

//	System.out.println("\nPhase angle is "
//			   + new Integer((int)(phaseAngle*180/Math.PI))
//			      .toString()
//			   + " degrees");

	//
	// Set the colors, if they're not already set:
	//
	if (moonColor == null) {
	    moonColor = Color.white;
	    drawingArea.setBackground(Color.black);
	    drawingArea.setForeground(Color.white);
	}

	if (earthshineColor == null) {
	    if (showEarthshine)
		earthshineColor = Color.gray;
	    else
		earthshineColor = drawingArea.getBackground();
	}

	//
	// and do the drawing:
	//
	int moonradius = (int)(Math.min(width, height) * .4);

        if (borderWidth > 0) {
            g.setColor(Color.white);
            g.fillRect(0, 0, width, height);
        }
    	g.setColor(drawingArea.getBackground());
    	g.fillRect(borderWidth, borderWidth,
    	           width-borderWidth*2, height-borderWidth*2);

    	int xcenter = width/2;
    	int ycenter = height/2;

      if (true)
      {
        // The phase angle is the angle sun-moon-earth,
        // so 0 = full phase, 180 = new.
        // What we're actually interested in for drawing purposes
        // is the position angle of the sunrise terminator,
        // which runs the opposite direction from the phase angle,
        // so we have to convert.
        double positionAngle = Math.PI - phaseAngle;
        if (positionAngle < 0.) positionAngle += 2.*Math.PI;
	//System.out.println("Position angle is "
        //                   + new Integer((int)(positionAngle*180/Math.PI))
	//		      .toString()
	//		   + " degrees");

       	// draw the whole moon disk, in moonColor:
    	g.setColor(moonColor);
    	g.fillOval(xcenter-moonradius, ycenter-moonradius,
		   moonradius*2, moonradius*2);

	// Okay, now fill in the dark part.
    	g.setColor(earthshineColor);

        double cosTerm = Math.cos(positionAngle);
        //if (cosTerm < 0) cosTerm = -cosTerm;
        double rsquared = moonradius*moonradius;
        int whichQuarter = ((int)(positionAngle*2./Math.PI) + 4) % 4;
        int j;
        //System.out.println("Phase angle: " + new Float(phaseAngle));
        for (j=0; j <= moonradius; ++j)
        {
            double rrf = Math.sqrt(rsquared - j*j);
            int rr = (int)(rrf + .5);
            int xx = (int)(rrf * cosTerm);
            //if (j == 1)
            //    System.out.println("xx = " + new Integer(xx).toString()
            //                       + " of " + new Integer(rr).toString());
            int x1 = xcenter - (whichQuarter < 2 ? rr : xx);
            int w = rr + xx + 1;
            g.fillRect(x1, ycenter-j, w, 1);
            g.fillRect(x1, ycenter+j, w, 1);
        }
        return;
      }

       	// draw the whole moon disk, in earthshineColor:
    	g.setColor(earthshineColor);
    	g.fillOval(xcenter-moonradius, ycenter-moonradius,
		   moonradius*2, moonradius*2);

	// Okay, now fill in the bright part with white.
    	g.setColor(moonColor);

	// phase angle crib notes:
	// 0/360 = full moon
	// 180 = new moon
	// 90 = right half bright (a guess)

	// We'll need the x coordinate of the terminator:
	int dxterm = Math.abs((int)(moonradius * Math.cos(phaseAngle)));

	// CASE 1: 0-90, dark only on left side:
	if (phaseAngle < Math.PI/2) {
	    // First, fill all of the right half:
	    g.fillArc(xcenter-moonradius, ycenter-moonradius,
		      moonradius*2, moonradius*2, -90, 180);
	    // Now fill part of the left half:
	    g.fillArc(xcenter-dxterm, ycenter-moonradius,
		      dxterm*2, moonradius*2, 90, 180);
	}

	// CASE 2: 90-180, light only on right side:
	else if (phaseAngle < Math.PI) {
	    // First, fill all of the right half:
	    g.fillArc(xcenter-moonradius, ycenter-moonradius,
		      moonradius*2, moonradius*2, -90, 180);
	    // Now overfill part of the right half with grey:
	    g.setColor(earthshineColor);
	    g.fillArc(xcenter-dxterm, ycenter-moonradius,
		      dxterm*2, moonradius*2, -90, 180);
	}

	// CASE 3: 180-270, light only on left side:
	else if (phaseAngle < Math.PI * 1.5) {
	    // First, fill the left half:
	    g.fillArc(xcenter-moonradius, ycenter-moonradius,
		      moonradius*2, moonradius*2, 90, 180);
	    // Now overfill part of the left half with grey:
	    g.setColor(earthshineColor);
	    g.fillArc(xcenter-dxterm, ycenter-moonradius,
		      dxterm*2, moonradius*2, 90, 180);
	}

	// CASE 4: 270-360, dark only on right side, dxterm positive:
	else {
	    // First, fill the left half:
	    g.fillArc(xcenter-moonradius, ycenter-moonradius,
		      moonradius*2, moonradius*2, 90, 180);
	    // Now fill part of the right half:
	    g.fillArc(xcenter-dxterm, ycenter-moonradius,
		      dxterm*2, moonradius*2, -90, 180);
	}
    }
}

